package de.fhdw.gaming.ipspiel23.tictactoe.strategy.calculator;

import java.util.Optional;

import de.fhdw.gaming.core.domain.GameException;
import de.fhdw.gaming.ipspiel23.gst.domain.ICalculatorKopplung;
import de.fhdw.gaming.ipspiel23.gst.domain.IKopplung;
import de.fhdw.gaming.ipspiel23.gst.domain.impl.GstKopplungsMoveCalculator;
import de.fhdw.gaming.ipspiel23.gst.strategies.impl.GstKopplungNegaMax;
import de.fhdw.gaming.ipspiel23.tictactoe.core.domain.TicTacToePlayer;
import de.fhdw.gaming.ipspiel23.tictactoe.core.domain.TicTacToeState;
import de.fhdw.gaming.ipspiel23.tictactoe.core.domain.TicTacToeStrategy;
import de.fhdw.gaming.ipspiel23.tictactoe.core.domain.impl.TicTacToeKopplungImpl;
import de.fhdw.gaming.ipspiel23.tictactoe.core.moves.TicTacToeMove;

/** 
 * Implementation of the {@link TicTacToeStrategy} interface.
 * It computes the next move by using the GstKopplungNegaMax algorithms.
 * This strategy is specifically designed for Tic-Tac-Toe games.
 * 
 * The calculation is performed by an instance of {@link ICalculatorKopplung} 
 * with a {@link GstKopplungNegaMax} algorithm.
 * The game state evaluation is done using an instance of {@link TicTacToeKopplungImpl}.
 * 
 * @author borkowitz
 */
public class TicTacToeCalculatorMoveStrategy implements TicTacToeStrategy {

    /**
     * The calculator used for computing the next move.
     */
    private final ICalculatorKopplung<TicTacToePlayer, TicTacToeState> calc;

    /**
     * The game state coupling used for evaluation and move calculation.
     */
    private final IKopplung<TicTacToePlayer, TicTacToeState> ticTacToeKopplung;

    /**
     * new TicTacToeCalculatorMoveStrategy Constructor.
     */
    public TicTacToeCalculatorMoveStrategy() {
        this.calc = new GstKopplungsMoveCalculator<>();
        this.ticTacToeKopplung = new TicTacToeKopplungImpl();
    }


    /**
     * Computes the next move for the given Tic-Tac-Toe game State.
     * 
     * @param gameId The ID of the game.
     * @param player The current player.
     * @param state The current game state.
     * @return An optional containing the next Tic-Tac-Toe move.
     * @throws GameException If an error occurs during the game.
     * @throws InterruptedException If the calculation is interrupted.
     */
    @Override
    public Optional<TicTacToeMove> computeNextMove(final int gameId, final TicTacToePlayer player, 
            final TicTacToeState state) throws GameException, InterruptedException {
        return Optional.of((TicTacToeMove) calc.calculateMove(
            this.ticTacToeKopplung, state, 500, new GstKopplungNegaMax<>()));
    }

    /**
     * Returns a string representation of this TicTacToeCalculatorMoveStrategy object.
     * 
     * @return A string representation of the object.
     */
    @Override
    public String toString() {
        return TicTacToeCalculatorMoveStrategy.class.getSimpleName();
    }
}
